#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2011  Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#
# gen.filters.rules/Source/_MatchesRepositoryFilter.py

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from ....lib import Source
from ....db import Database


_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .. import MatchesFilterBase


# -------------------------------------------------------------------------
# "Sources which reference a repository by selection"
# -------------------------------------------------------------------------
class MatchesRepositoryFilter(MatchesFilterBase):
    """Sources which reference the selected repository"""

    labels = [_("Repository filter name:")]
    name = _("Sources with repository reference matching the <repository filter>")
    description = _(
        "Matches sources with a repository reference that match a certain\n"
        "repository filter"
    )
    category = _("General filters")

    # we want to have this filter show repository filters
    namespace = "Repository"

    def prepare(self, db: Database, user):
        MatchesFilterBase.prepare(self, db, user)
        self.MRF_filt = self.find_filter()

    def apply_to_one(self, db: Database, object: Source) -> bool:
        if self.MRF_filt is None:
            return False

        repolist = [x.ref for x in object.reporef_list]
        for repohandle in repolist:
            # check if repo in repository filter
            repo = db.get_repository_from_handle(repohandle)
            if self.MRF_filt.apply_to_one(db, repo):
                return True
        return False



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2007-2008  Brian G. Matherly
# Copyright (C) 2008       Gary Burton
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

"""
Option class representing a file destination.
"""

# -------------------------------------------------------------------------
#
# gramps modules
#
# -------------------------------------------------------------------------
from . import StringOption


# -------------------------------------------------------------------------
#
# DestinationOption class
#
# -------------------------------------------------------------------------
class DestinationOption(StringOption):
    """
    This class describes an option that specifies a destination file or path.
    The destination can be a directory or a file. If the destination is a file,
    the extension can be specified.
    """

    __signals__ = {"options-changed": None}

    def __init__(self, label, value):
        """
        :param label: A friendly label to be applied to this option.
            Example: "File Name"
        :type label: string
        :param value: A default destination for this option.
            Example: "/home/username/Desktop/"
            Example: "/home/username/Desktop/report.pdf"
        :type value: string
        :param is_directory: Specifies whether the destination is a directory
                             or a file.
        :type is_directory: bool
        :return: nothing
        """
        StringOption.__init__(self, label, value)
        self.__is_directory = False
        self.__extension = ""

    def set_directory_entry(self, is_directory):
        """
        :param is_directory: Specifies whether the destination is a directory
                             or a file.
        :type is_directory: bool
        :return: nothing
        """
        self.__is_directory = is_directory
        self.emit("options-changed")

    def get_directory_entry(self):
        """
        :return: True if the destination is a directory. False if the
                 destination is a file.
        :rtype: bool
        """
        return self.__is_directory

    def set_extension(self, extension):
        """
        :param extension: Specifies the extension for the destination file.
        :type extension: str
        :return: nothing
        """
        self.__extension = extension

    def get_extension(self):
        """
        :return: The extension for the destination file.
        """
        return self.__extension



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
# Copyright (C) 2008       Gary Burton
# Copyright (C) 2010,2015  Nick Hall
# Copyright (C) 2017-      Serge Noiraud
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# gtk
#
# -------------------------------------------------------------------------
from gi.repository import Gtk

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ... import widgets
from gramps.gen.lib import Place, PlaceType
from .. import build_filter_model
from . import SidebarFilter
from gramps.gen.filters import GenericFilterFactory, rules
from gramps.gen.filters.rules.place import (
    RegExpIdOf,
    HasData,
    IsEnclosedBy,
    HasTag,
    HasNoteRegexp,
    WithinArea,
    MatchesFilter,
)

GenericPlaceFilter = GenericFilterFactory("Place")


# -------------------------------------------------------------------------
#
# PlaceSidebarFilter class
#
# -------------------------------------------------------------------------
class PlaceSidebarFilter(SidebarFilter):
    def __init__(self, dbstate, uistate, clicked):
        self.clicked_func = clicked

        self.filter_id = widgets.BasicEntry()
        self.filter_name = widgets.BasicEntry()
        self.filter_place = Place()
        self.filter_place.set_type((PlaceType.CUSTOM, ""))
        self.ptype = Gtk.ComboBox(has_entry=True)
        self.dbstate = dbstate
        if dbstate.is_open():
            self.custom_types = dbstate.db.get_place_types()
        else:
            self.custom_types = []

        self.place_menu = widgets.MonitoredDataType(
            self.ptype,
            self.filter_place.set_type,
            self.filter_place.get_type,
            False,  # read-only
            self.custom_types,
        )
        self.filter_code = widgets.BasicEntry()
        self.filter_enclosed = widgets.PlaceEntry(dbstate, uistate, [])
        self.filter_note = widgets.BasicEntry()
        self.filter_within = widgets.PlaceWithin(dbstate, uistate, [])

        self.filter_regex = Gtk.CheckButton(label=_("Use regular expressions"))
        self.sensitive_regex = Gtk.CheckButton(label=_("Case sensitive"))
        self.tag = Gtk.ComboBox()
        self.generic = Gtk.ComboBox()

        SidebarFilter.__init__(self, dbstate, uistate, "Place")

    def create_widget(self):
        cell = Gtk.CellRendererText()
        cell.set_property("width", self._FILTER_WIDTH)
        cell.set_property("ellipsize", self._FILTER_ELLIPSIZE)
        self.generic.pack_start(cell, True)
        self.generic.add_attribute(cell, "text", 0)
        self.on_filters_changed("Place")

        cell = Gtk.CellRendererText()
        cell.set_property("width", self._FILTER_WIDTH)
        cell.set_property("ellipsize", self._FILTER_ELLIPSIZE)
        self.tag.pack_start(cell, True)
        self.tag.add_attribute(cell, "text", 0)

        self.add_text_entry(_("ID"), self.filter_id)
        self.add_text_entry(_("Name"), self.filter_name)
        self.add_entry(_("Type"), self.ptype)
        self.add_text_entry(_("Code"), self.filter_code)
        self.add_text_entry(_("Enclosed By"), self.filter_enclosed)
        self.add_text_entry(_("Within"), self.filter_within)
        self.add_text_entry(_("Note"), self.filter_note)
        self.add_entry(_("Tag"), self.tag)
        self.add_filter_entry(_("Custom filter"), self.generic)
        self.add_regex_entry(self.filter_regex)
        self.add_regex_case(self.sensitive_regex)

    def clear(self, obj):
        self.filter_id.set_text("")
        self.filter_name.set_text("")
        self.filter_code.set_text("")
        self.filter_enclosed.set_text("")
        self.filter_note.set_text("")
        self.filter_within.set_value("", 0)
        self.ptype.get_child().set_text("")
        self.tag.set_active(0)
        self.generic.set_active(0)

    def get_filter(self):
        gid = str(self.filter_id.get_text()).strip()
        name = str(self.filter_name.get_text()).strip()
        ptype = self.filter_place.get_type().xml_str()
        code = str(self.filter_code.get_text()).strip()
        enclosed = str(self.filter_enclosed.get_text()).strip()
        note = str(self.filter_note.get_text()).strip()
        within = self.filter_within.get_value()
        regex = self.filter_regex.get_active()
        usecase = self.sensitive_regex.get_active()
        tag = self.tag.get_active() > 0
        gen = self.generic.get_active() > 0

        empty = not (
            gid
            or name
            or ptype
            or code
            or enclosed
            or note
            or regex
            or within[0]
            or tag
            or gen
        )
        if empty:
            generic_filter = None
        else:
            generic_filter = GenericPlaceFilter()
            if gid:
                rule = RegExpIdOf([gid], use_regex=regex, use_case=usecase)
                generic_filter.add_rule(rule)

            if enclosed:
                rule = IsEnclosedBy([enclosed, "0"])
                generic_filter.add_rule(rule)

            rule = HasData([name, ptype, code], use_regex=regex, use_case=usecase)
            generic_filter.add_rule(rule)

            if note:
                rule = HasNoteRegexp([note], use_regex=regex, use_case=usecase)
                generic_filter.add_rule(rule)

            if within and within[0] > 0 and self.dbstate.is_open():
                rule = WithinArea([None, within[0], within[1]])
                active_ref = self.uistate.get_active("Place")
                if active_ref:
                    place = self.dbstate.db.get_place_from_handle(active_ref)
                    gid = place.get_gramps_id()
                    rule = WithinArea([gid, within[0], within[1]])
                generic_filter.add_rule(rule)

            # check the Tag
            if tag:
                model = self.tag.get_model()
                node = self.tag.get_active_iter()
                attr = model.get_value(node, 0)
                rule = HasTag([attr])
                generic_filter.add_rule(rule)

            if self.generic.get_active() != 0:
                model = self.generic.get_model()
                node = self.generic.get_active_iter()
                obj = str(model.get_value(node, 0))
                rule = MatchesFilter([obj])
                generic_filter.add_rule(rule)

        return generic_filter

    def on_filters_changed(self, name_space):
        if name_space == "Place":
            all_filter = GenericPlaceFilter()
            all_filter.set_name(_("None"))
            all_filter.add_rule(rules.place.AllPlaces([]))
            self.generic.set_model(build_filter_model("Place", [all_filter]))
            self.generic.set_active(0)

    def on_tags_changed(self, tag_list):
        """
        Update the list of tags in the tag filter.
        """
        model = Gtk.ListStore(str)
        model.append(("",))
        for tag_name in tag_list:
            model.append((tag_name,))
        self.tag.set_model(model)
        self.tag.set_active(0)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
# Copyright (C) 2007       Brian G. Matherly
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

"""
Package providing filter rules for Gramps.
"""

from ._allnotes import AllNotes
from ._hasidof import HasIdOf
from ._regexpidof import RegExpIdOf
from ._matchesregexpof import MatchesRegexpOf
from ._matchessubstringof import MatchesSubstringOf
from ._hasreferencecountof import HasReferenceCountOf
from ._noteprivate import NotePrivate
from ._matchesfilter import MatchesFilter
from ._hasnote import HasNote
from ._changedsince import ChangedSince
from ._hastag import HasTag
from ._hastype import HasType

editor_rule_list = [
    AllNotes,
    HasIdOf,
    RegExpIdOf,
    HasNote,
    MatchesRegexpOf,
    HasReferenceCountOf,
    NotePrivate,
    MatchesFilter,
    ChangedSince,
    HasTag,
    HasType,
]



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2003  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#


# ------------------------------------------------------------------------
#
#
#
# ------------------------------------------------------------------------
class TabbedDoc:
    def __init__(self, columns):
        self.columns = columns
        self.name = ""

    def creator(self, name):
        self.name = name

    def open(self, filename):
        pass

    def close(self):
        pass

    def start_page(self):
        pass

    def end_page(self):
        pass

    def start_paragraph(self):
        pass

    def end_paragraph(self):
        pass

    def start_table(self):
        pass

    def end_table(self):
        pass

    def start_row(self):
        pass

    def end_row(self):
        pass

    def write_cell(self, text):
        pass



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ....utils.alive import probably_alive
from .. import Rule
from ....datehandler import parser


# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from ....lib import Person
from ....db import Database


# -------------------------------------------------------------------------
# "People probably alive"
# -------------------------------------------------------------------------
class ProbablyAlive(Rule):
    """People probably alive"""

    labels = [_("On date:")]
    name = _("People probably alive")
    description = _("Matches people without indications of death that are not too old")
    category = _("General filters")

    def prepare(self, db: Database, user):
        try:
            self.current_date = parser.parse(str(self.list[0]))
        except:
            self.current_date = None

    def apply_to_one(self, db, person: Person) -> bool:
        return probably_alive(person, db, self.current_date)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2003-2005  Donald N. Allingham
# Copyright (C) 2008       Brian G. Matherly
# Copyright (C) 2005-2012  Julio Sanchez
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any latr version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#
# -------------------------------------------------------------------------
"""
Spanish-specific classes for relationships.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------

from gramps.gen.lib import Person

MALE = Person.MALE
FEMALE = Person.FEMALE
UNKNOWN = Person.UNKNOWN
import gramps.gen.relationship

# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------

_level_name_male = [
    "",
    "primero",
    "segundo",
    "tercero",
    "cuarto",
    "quinto",
    "sexto",
    "séptimo",
    "octavo",
    "noveno",
    "décimo",
    "undécimo",
    "duodécimo",
    "decimotercero",
    "decimocuarto",
    "decimoquinto",
    "decimosexto",
    "decimoséptimo",
    "decimoctavo",
    "decimonono",
    "vigésimo",
]

# Short forms (in apocope) used before names
_level_name_male_a = [
    "",
    "primer",
    "segundo",
    "tercer",
    "cuarto",
    "quinto",
    "sexto",
    "séptimo",
    "octavo",
    "noveno",
    "décimo",
    "undécimo",
    "duodécimo",
    "decimotercer",
    "decimocuarto",
    "decimoquinto",
    "decimosexto",
    "decimoséptimo",
    "decimoctavo",
    "decimonono",
    "vigésimo",
]

_level_name_female = [
    "",
    "primera",
    "segunda",
    "tercera",
    "cuarta",
    "quinta",
    "sexta",
    "séptima",
    "octava",
    "novena",
    "décima",
    "undécima",
    "duodécima",
    "decimotercera",
    "decimocuarta",
    "decimoquinta",
    "decimosexta",
    "decimoséptima",
    "decimoctava",
    "decimonona",
    "vigésima",
]

_level_name_plural = [
    "",
    "primeros",
    "segundos",
    "terceros",
    "cuartos",
    "quintos",
    "sextos",
    "séptimos",
    "octavos",
    "novenos",
    "décimos",
    "undécimos",
    "duodécimos",
    "decimoterceros",
    "decimocuartos",
    "decimoquintos",
    "decimosextos",
    "decimoséptimos",
    "decimoctavos",
    "decimononos",
    "vigésimos",
]

# This plugin tries to be flexible and expect little from the following
# tables.  Ancestors are named from the list for the first generations.
# When this list is not enough, ordinals are used based on the same idea,
# i.e. bisabuelo is 'segundo abuelo' and so on, that has been the
# traditional way in Spanish.  When we run out of ordinals we resort to
# N-ésimo notation, that is sort of understandable if in context.
# 'trastatarabuelo' is not in DRAE, but is well known
_parents_level = [
    "",
    "padres",
    "abuelos",
    "bisabuelos",
    "tatarabuelos",
    "trastatarabuelos",
]

_father_level = [
    "",
    "padre%(inlaw)s",
    "abuelo%(inlaw)s",
    "bisabuelo%(inlaw)s",
    "tatarabuelo%(inlaw)s",
    "trastatarabuelo%(inlaw)s",
]

_mother_level = [
    "",
    "madre%(inlaw)s",
    "abuela%(inlaw)s",
    "bisabuela%(inlaw)s",
    "tatarabuela%(inlaw)s",
    "trastatarabuela%(inlaw)s",
]

# step-relationships can't be handled as in English
# Notice that the traditional lack of divorce in Catholic, Spanish-speaking, countries has resulted
# in a scarcity of terms to describe these relationships since only death of a spouse would let the
# other marry again. Divorce is common now, so these relationships abound, but history has left us
# without support in the language. So, in this case, we will be more liberal than in other cases and
# or coin a few new words or accept others that seem to have some use, but always patterned
# after the style of the well-documented cases, so that users can intuitively guess their meaning.
# Notice that "that relationship does not exist in Spanish" is not a valid objection. Once the Gramps
# core has computed a relationship, it *has* to be named *somehow*. The only alternative is to change
# the Gramps core so that it does not find relationships that cannot be named in Spanish.

_step_father_level = ["", "padrastro%(inlaw)s", "abuelastro%(inlaw)s"]

_step_mother_level = ["", "madrastra%(inlaw)s", "abuelastra%(inlaw)s"]

# Higher-order terms (after trastatarabuelo) on this list are not standard,
# but then there is no standard naming scheme at all for this in Spanish.
# Check http://www.genealogia-es.com/guia3.html that echoes a proposed
# scheme that has got some reception in the Spanish-language genealogy
# community. Uncomment these names if you want to use them.
# _parents_level = [ "", "padres", "abuelos", "bisabuelos", "tatarabuelos",
#                   "trastatarabuelos", "pentabuelos", "hexabuelos",
#                   "heptabuelos", "octabuelos", "eneabuelos", "decabuelos"]
# _father_level = [ "", "padre", "abuelo", "bisabuelo", "tatarabuelo",
#                  "trastatarabuelo", "pentabuelo", "hexabuelo",
#                  "heptabuelo", "octabuelo", "eneabuelo", "decabuelo"]
# _mother_level = [ "", "madre", "abuela", "bisabuela", "tatarabuela",
#                  "trastatarabuela", "pentabuela", "hexabuela",
#                  "heptabuela", "octabuela", "eneabuela", "decabuela"]

# DRAE defines cuadrinieto as well, with the same meaning as chozno
# trastataranieto is in use too, but is not in DRAE
# DRAE also registers bizchozno and bischozno, but prefers bichozno
_son_level = [
    "",
    "hijo%(inlaw)s",
    "nieto%(inlaw)s",
    "bisnieto%(inlaw)s",
    "tataranieto%(inlaw)s",
    "chozno%(inlaw)s",
    "bichozno%(inlaw)s",
]

# Though "abuelastro" is in DRAE, "nietastro" isn't
_step_son_level = ["", "hijastro%(inlaw)s", "nietastro%(inlaw)s"]

_daughter_level = [
    "",
    "hija%(inlaw)s",
    "nieta%(inlaw)s",
    "bisnieta%(inlaw)s",
    "tataranieta%(inlaw)s",
    "chozna%(inlaw)s",
    "bichozna%(inlaw)s",
]

_step_daughter_level = ["", "hijastra%(inlaw)s", "nietastra%(inlaw)s"]

_sister_level = [
    "",
    "hermana%(inlaw)s",
    "tía%(inlaw)s",
    "tía abuela%(inlaw)s",
    "tía bisabuela%(inlaw)s",
    "tía tatarabuela%(inlaw)s",
]

# Tiastro/tiastra aren't in DRAE
_step_sister_level = [
    "",
    "hermanastra%(inlaw)s",
    "tiastra%(inlaw)s",
    "tía abuelastra%(inlaw)s",
]

_brother_level = [
    "",
    "hermano%(inlaw)s",
    "tío%(inlaw)s",
    "tío abuelo%(inlaw)s",
    "tío bisabuelo%(inlaw)s",
    "tío tatarabuelo%(inlaw)s",
]

_step_brother_level = [
    "",
    "hermanastro%(inlaw)s",
    "tiastro%(inlaw)s",
    "tío abuelastro%(inlaw)s",
]

_nephew_level = [
    "",
    "sobrino%(inlaw)s",
    "sobrino nieto%(inlaw)s",
    "sobrino bisnieto%(inlaw)s",
    "sobrino tataranieto%(inlaw)s",
    "sobrino chozno%(inlaw)s",
    "sobrino bichozno%(inlaw)s",
]

# Nether are sobrinastro/sobrinastra
_step_nephew_level = ["", "sobrinastro%(inlaw)s", "sobrino nietastro%(inlaw)s"]

_niece_level = [
    "",
    "sobrina%(inlaw)s",
    "sobrina nieta%(inlaw)s",
    "sobrina bisnieta%(inlaw)s",
    "sobrina tataranieta%(inlaw)s",
    "sobrina chozna%(inlaw)s",
    "sobrina bichozna%(inlaw)s",
]

_step_niece_level = ["", "sobrinastra%(inlaw)s", "sobrina nietastra%(inlaw)s"]

_children_level = [
    "",
    "hijos",
    "nietos",
    "bisnietos",
    "tataranietos",
    "choznos",
    "bichoznos",
]

_siblings_level = [
    "",
    "hermanos/as",
    "tíos/tías",
    "tíos abuelos/tías abuelas",
    "tíos bisabuelos/tías bisabuelas",
    "tíos tatarabuelos/tías tatarabuelas",
    "tíos trastatarabuelos/tías trastatarabuelas",
]

_nephews_nieces_level = [
    "",
    "hermanos/as",
    "sobrinos/as",
    "sobrinos nietos/sobrinas nietas",
    "sobrinos bisnietos/sobrinas bisnietas",
    "sobrinos tataranietos/sobrinas tataranietas",
    "sobrinos choznos/sobrinas choznas",
    "sobrinos bichoznos/sobrinas bichoznas",
]


# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------
class RelationshipCalculator(gramps.gen.relationship.RelationshipCalculator):
    """
    RelationshipCalculator Class
    """

    def __init__(self):
        gramps.gen.relationship.RelationshipCalculator.__init__(self)

    def _get_step_father(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_father_level):
            return _step_father_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_male_a):
            return "%s abuelastro%s" % (_level_name_male_a[level - 1], inlaw)
        else:
            return "%d-ésimo abuelastro%s" % (level - 1, inlaw)

    def _get_father(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_father(level, inlaw)
        if inlaw and level == 1:
            return "suegro"
        if level < len(_father_level):
            return _father_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_male_a):
            return "%s abuelo%s" % (_level_name_male_a[level - 1], inlaw)
        else:
            return "%d-ésimo abuelo%s" % (level - 1, inlaw)

    def _get_step_son(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_son_level):
            return _step_son_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_male_a):
            return "%s nietastro%s" % (_level_name_male_a[level - 1], inlaw)
        else:
            return "%d-ésimo nietastro%s" % (level - 1, inlaw)

    def _get_son(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_son(level, inlaw)
        if inlaw and level == 1:
            return "yerno"
        if level < len(_son_level):
            return _son_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_male_a):
            return "%s nieto%s" % (_level_name_male_a[level - 1], inlaw)
        else:
            return "%d-ésimo nieto%s" % (level - 1, inlaw)

    def _get_step_mother(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_mother_level):
            return _step_mother_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_female):
            return "%s abuelastra%s" % (_level_name_female[level - 1], inlaw)
        else:
            return "%d-ésima abuelastra%s" % (level - 1, inlaw)

    def _get_mother(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_mother(level, inlaw)
        if inlaw and level == 1:
            return "suegra"
        if level < len(_mother_level):
            return _mother_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_female):
            return "%s abuela%s" % (_level_name_female[level - 1], inlaw)
        else:
            return "%d-ésima abuela%s" % (level - 1, inlaw)

    def _get_step_daughter(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_daughter_level):
            return _step_daughter_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_female):
            return "%s nietastra%s" % (_level_name_female[level - 1], inlaw)
        else:
            return "%d-ésima nietastra%s" % (level - 1, inlaw)

    def _get_daughter(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_daughter(level, inlaw)
        if inlaw and level == 1:
            return "nuera"
        if level < len(_daughter_level):
            return _daughter_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_female):
            return "%s nieta%s" % (_level_name_female[level - 1], inlaw)
        else:
            return "%d-ésima nieta%s" % (level - 1, inlaw)

    def _get_parent_unknown(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        return "%s o %s" % (
            self._get_father(level, step, inlaw),
            self._get_mother(level, step, inlaw),
        )

    def _get_child_unknown(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        return "%s o %s" % (
            self._get_son(level, step, inlaw),
            self._get_daughter(level, step, inlaw),
        )

    def _get_step_aunt(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_sister_level):
            return _step_sister_level[level] % {"inlaw": inlaw}
        elif (level - 2) < len(_level_name_female):
            return "%s tía abuelastra%s" % (_level_name_female[level - 2], inlaw)
        else:
            return "%d-ésima tia abuelastra%s" % (level - 2, inlaw)

    def _get_aunt(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_aunt(level, inlaw)
        if inlaw and level == 1:
            return "cuñada"
        if level < len(_sister_level):
            return _sister_level[level] % {"inlaw": inlaw}
        elif (level - 2) < len(_level_name_female):
            return "%s tía abuela%s" % (_level_name_female[level - 2], inlaw)
        else:
            return "%d-ésima tía abuela%s" % (level - 2, inlaw)

    def _get_distant_aunt(self, level, step, inlaw):
        if step:
            base = "tiastra"
        else:
            base = "tía"
        if level < len(_level_name_female):
            return "%s %s" % (base, _level_name_female[level])
        else:
            return "%s %d-ésima" % (base, level)

    def _get_step_uncle(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_brother_level):
            return _step_brother_level[level] % {"inlaw": inlaw}
        elif (level - 2) < len(_level_name_male_a):
            return "%s tío abuelastro%s" % (_level_name_male_a[level - 2], inlaw)
        else:
            return "%d-ésimo tío abuelastro%s" % (level - 2, inlaw)

    def _get_uncle(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_uncle(level, inlaw)
        if inlaw and level == 1:
            return "cuñado"
        if level < len(_brother_level):
            return _brother_level[level] % {"inlaw": inlaw}
        elif (level - 2) < len(_level_name_male_a):
            return "%s tío abuelo%s" % (_level_name_male_a[level - 2], inlaw)
        else:
            return "%d-ésimo tío abuelo%s" % (level - 2, inlaw)

    def _get_distant_uncle(self, level, step="", inlaw=""):
        if step:
            base = "tiastro"
        else:
            base = "tío"
        if level < len(_level_name_male):
            return "%s %s" % (base, _level_name_male[level])
        else:
            return "%s %d-ésimo" % (base, level)

    def _get_step_nephew(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_nephew_level):
            return _step_nephew_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_male_a):
            return "%s tío sobrinastro%s" % (_level_name_male_a[level - 1], inlaw)
        else:
            return "%d-ésimo tío sobrinastro%s" % (level - 1, inlaw)

    def _get_nephew(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_nephew(level, inlaw)
        if level < len(_nephew_level):
            return _nephew_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_male_a):
            return "%s sobrino nieto%s" % (_level_name_male_a[level - 1], inlaw)
        else:
            return "%d-ésimo sobrino nieto%s" % (level - 1, inlaw)

    def _get_distant_nephew(self, level, step, inlaw):
        if step:
            base = "sobrinastro"
        else:
            base = "sobrino"
        if level < len(_level_name_male):
            return "%s %s" % (base, _level_name_male[level])
        else:
            return "%s %d-ésimo" % (base, level)

    def _get_step_niece(self, level, inlaw=""):
        """Internal spanish method to create relation string"""
        if level < len(_step_niece_level):
            return _step_niece_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_female):
            return "%s tía sobrinastra%s" % (_level_name_female[level - 1], inlaw)
        else:
            return "%d-ésima tía sobrinastra%s" % (level - 1, inlaw)

    def _get_niece(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        if step:
            return self._get_step_niece(level, inlaw)
        if level < len(_niece_level):
            return _niece_level[level] % {"inlaw": inlaw}
        elif (level - 1) < len(_level_name_female):
            return "%s sobrina nieta%s" % (_level_name_female[level - 1], inlaw)
        else:
            return "%d-ésima sobrina nieta%s" % (level - 1, inlaw)

    def _get_distant_niece(self, level, step, inlaw):
        if step:
            base = "sobrinastra"
        else:
            base = "sobrina"
        if level < len(_level_name_female):
            return "%s %s" % (base, _level_name_female[level])
        else:
            return "%s %d-ésima" % (base, level)

    def _get_male_cousin(
        self, level, removed, lower=False, step="", inlaw="", gender_c=UNKNOWN
    ):
        """Internal spanish method to create relation string"""
        # primastro is an invention and is not backed by DRAE
        if step:
            prim = "primastro"
        else:
            prim = "primo"
        if removed == 0:
            if level == 1:
                return "%s hermano%s" % (prim, inlaw)
            elif level < len(_level_name_male):
                return "%s %s%s" % (prim, _level_name_male[level], inlaw)
            else:
                return "%s %d-ésimo%s" % (prim, level, inlaw)
        elif removed > 0 and lower:
            if gender_c == MALE:
                return "%s de un %s" % (
                    self._get_son(removed, step, inlaw),
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                )
            elif gender_c == FEMALE:
                return "%s de una %s" % (
                    self._get_son(removed, step, inlaw),
                    self._get_female_cousin(level, 0, lower, step, inlaw, gender_c),
                )
            else:
                return "%s de un %s" % (
                    self._get_son(removed, step, inlaw),
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                )
        elif removed > 0 and not lower:
            if gender_c == MALE:
                return "%s de un %s" % (
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                    self._get_father(removed, step, inlaw),
                )
            elif gender_c == FEMALE:
                return "%s de una %s" % (
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                    self._get_mother(removed, step, inlaw),
                )
            else:
                return "%s de un %s" % (
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                    self._get_father(removed, step, inlaw),
                )

        else:
            return "%s %scousin%s (%d-%d)" % (
                _level_name_male[level],
                step,
                inlaw,
                removed,
                lower,
            )

    def _get_female_cousin(
        self, level, removed, lower=False, step="", inlaw="", gender_c=UNKNOWN
    ):
        """Internal spanish method to create relation string"""
        # primastra is an invention and is not real Spanish
        if step:
            prim = "primastra"
        else:
            prim = "prima"
        if removed == 0:
            if level == 1:
                return "%s hermana%s" % (prim, inlaw)
            elif level < len(_level_name_male):
                return "%s %s%s" % (prim, _level_name_female[level], inlaw)
            else:
                return "%s %d-ésima%s" % (prim, level, inlaw)
        elif removed > 0 and lower:
            if gender_c == MALE:
                return "%s de un %s" % (
                    self._get_daughter(removed, step, inlaw),
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                )
            elif gender_c == FEMALE:
                return "%s de una %s" % (
                    self._get_daughter(removed, step, inlaw),
                    self._get_female_cousin(level, 0, lower, step, inlaw, gender_c),
                )
            else:
                return "%s de un %s" % (
                    self._get_daughter(removed, step, inlaw),
                    self._get_male_cousin(level, 0, lower, step, inlaw, gender_c),
                )
        elif removed > 0 and not lower:
            if gender_c == MALE:
                return "%s de un %s" % (
                    self._get_female_cousin(level, 0, lower, step, inlaw, gender_c),
                    self._get_father(removed, step, inlaw),
                )
            elif gender_c == FEMALE:
                return "%s de una %s" % (
                    self._get_female_cousin(level, 0, lower, step, inlaw, gender_c),
                    self._get_mother(removed, step, inlaw),
                )
            else:
                return "%s de un %s" % (
                    self._get_female_cousin(level, 0, lower, step, inlaw, gender_c),
                    self._get_father(removed, step, inlaw),
                )

        else:
            return "%s %sprima%s (%d-%d)" % (
                _level_name_female[level],
                step,
                inlaw,
                removed,
                lower,
            )

    def _get_sibling(self, level, step="", inlaw=""):
        """Internal spanish method to create relation string"""
        # TBC: inlaw is inflicted, it is probably better to do away with this method
        # and do both calls from the caller (would need inlaw_MALE and inlaw_FEMALE,
        # but is feasible
        return "%s o %s" % (
            self._get_uncle(level, step, inlaw),
            self._get_aunt(level, step, inlaw),
        )

    def get_plural_relationship_string(
        self,
        Ga,
        Gb,
        reltocommon_a="",
        reltocommon_b="",
        only_birth=True,
        in_law_a=False,
        in_law_b=False,
    ):
        """Spanish version of method to create relation string - check relationship.py"""

        rel_str = "parientes lejanos"
        if Ga == 0:
            # These are descendants
            if Gb < len(_children_level):
                rel_str = _children_level[Gb]
            elif (Gb - 1) < len(_level_name_plural):
                rel_str = "%s nietos" % (_level_name_plural[Gb - 1])
            else:
                rel_str = "%d-ésimos nietos" % (Gb - 1)
        elif Gb == 0:
            # These are parents/grand parents
            if Ga < len(_parents_level):
                rel_str = _parents_level[Ga]
            elif (Ga - 1) < len(_level_name_plural):
                rel_str = "%s abuelos" % (_level_name_plural[Ga - 1])
            else:
                rel_str = "%d-ésimos abuelos" % (Ga - 1)
        elif Gb == 1:
            # These are siblings/aunts/uncles
            if Ga < len(_siblings_level):
                rel_str = _siblings_level[Ga]
            elif (Ga - 1) < len(_level_name_plural):
                rel_str = "%s tíos abuelos" % (_level_name_plural[Ga - 1])
            else:
                rel_str = "%s-ésimos tíos abuelos" % (Ga - 1)
        elif Ga == 1:
            # These are nieces/nephews
            if Gb < len(_nephews_nieces_level):
                rel_str = _nephews_nieces_level[Gb]
            elif (Gb - 1) < len(_level_name_plural):
                rel_str = "%s sobrinos nietos" % (_level_name_plural[Gb - 1])
            else:
                rel_str = "%s-ésimos sobrinos nietos" % (Gb - 1)
        elif Ga > 1 and Ga == Gb:
            # These are cousins in the same generation
            if Ga == 2:
                rel_str = "primos hermanos"
            elif (Ga - 1) < len(_level_name_plural):
                rel_str = "primos %s" % (_level_name_plural[Ga - 1])
            else:
                rel_str = "primos %d-ésimos" % (Ga - 1)
        elif Ga == Gb + 1:
            # These are distant uncles/aunts
            if Gb < len(_level_name_plural):
                rel_str = "tíos %s" % (_level_name_plural[Gb])
            else:
                rel_str = "tíos %d-ésimos" % (Gb)
        elif Ga + 1 == Gb:
            # These are distant nephews/nieces
            if Gb - 1 < len(_level_name_plural):
                rel_str = "sobrinos %s" % (_level_name_plural[Gb - 1])
            else:
                rel_str = "sobrinos %d-ésimos" % (Gb - 1)
        elif Ga > 1 and Ga > Gb:
            # These are cousins in different generations with the second person
            # being in a higher generation from the common ancestor than the
            # first person.
            rel_str = "%s de los %s" % (
                self.get_plural_relationship_string(0, Gb),
                self.get_plural_relationship_string(Ga, 0),
            )
        elif Gb > 1 and Gb > Ga:
            # These are cousins in different generations with the second person
            # being in a lower generation from the common ancestor than the
            # first person.
            rel_str = "%s de los %s" % (
                self.get_plural_relationship_string(0, Gb),
                self.get_plural_relationship_string(Ga, 0),
            )

        if in_law_b == True:
            rel_str = "cónyuges de los %s" % rel_str

        return rel_str

    def get_single_relationship_string(
        self,
        Ga,
        Gb,
        gender_a,
        gender_b,
        reltocommon_a,
        reltocommon_b,
        only_birth=True,
        in_law_a=False,
        in_law_b=False,
    ):
        """Spanish version of method to create relation string - check relationship.py"""

        if only_birth:
            step = ""
        else:
            step = self.STEP

        if in_law_a or in_law_b:
            if gender_b == FEMALE:
                inlaw = " política"
            else:
                inlaw = " político"
        else:
            inlaw = ""

        rel_str = "%spariente%s lejano" % (step, inlaw)

        if Ga == 0:
            # b is descendant of a
            if Gb == 0:
                rel_str = "la misma persona"
            elif gender_b == MALE:
                rel_str = self._get_son(Gb, step, inlaw)
            elif gender_b == FEMALE:
                rel_str = self._get_daughter(Gb, step, inlaw)
            else:
                rel_str = self._get_child_unknown(Gb, step, inlaw)
        elif Gb == 0:
            # b is parents/grand parent of a
            if gender_b == MALE:
                rel_str = self._get_father(Ga, step, inlaw)
            elif gender_b == FEMALE:
                rel_str = self._get_mother(Ga, step, inlaw)
            else:
                rel_str = self._get_parent_unknown(Ga, step, inlaw)
        elif Gb == 1:
            # b is sibling/aunt/uncle of a
            if gender_b == MALE:
                rel_str = self._get_uncle(Ga, step, inlaw)
            elif gender_b == FEMALE:
                rel_str = self._get_aunt(Ga, step, inlaw)
            else:
                rel_str = self._get_sibling(Ga, step, inlaw)
        elif Ga == 1:
            # b is niece/nephew of a
            if gender_b == MALE:
                rel_str = self._get_nephew(Gb - 1, step, inlaw)
            elif gender_b == FEMALE:
                rel_str = self._get_niece(Gb - 1, step, inlaw)
            else:
                rel_str = "%s o %s" % (
                    self._get_nephew(Gb - 1, step, inlaw),
                    self._get_niece(Gb - 1, step, inlaw),
                )
        elif Ga == Gb:
            # a and b cousins in the same generation
            if gender_b == MALE:
                rel_str = self._get_male_cousin(
                    Ga - 1, 0, lower=False, step=step, inlaw=inlaw
                )
            elif gender_b == FEMALE:
                rel_str = self._get_female_cousin(
                    Ga - 1, 0, lower=False, step=step, inlaw=inlaw
                )
            else:
                rel_str = "%s o %s" % (
                    self._get_male_cousin(Ga - 1, 0, step=step, inlaw=inlaw),
                    self._get_female_cousin(Ga - 1, 0, step=step, inlaw=inlaw),
                )
        elif Ga == Gb + 1:
            if gender_b == Person.MALE:
                rel_str = self._get_distant_uncle(Gb, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_distant_aunt(Gb, step, inlaw)
            else:
                rel_str = "%s o %s" % (
                    self._get_distant_uncle(Gb, step, inlaw),
                    self._get_distant_aunt(Gb, step, inlaw),
                )
        elif Ga + 1 == Gb:
            if gender_b == Person.MALE:
                rel_str = self._get_distant_nephew(Gb - 1, step, inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = self._get_distant_niece(Gb - 1, step, inlaw)
            else:
                rel_str = "%s o %s" % (
                    self._get_distant_nephew(Gb - 1, step, inlaw),
                    self._get_distant_niece(Gb - 1, step, inlaw),
                )
        elif Ga > Gb:
            # These are cousins in different generations with the second person
            # being in a higher generation from the common ancestor than the
            # first person.
            # We need to know the gender of the ancestor of the first person who is on
            # the same generation as the other person
            if reltocommon_a[Ga - Gb - 1] == "f":
                gender_c = MALE
            elif reltocommon_a[Ga - Gb - 1] == "m":
                gender_c = FEMALE
            else:
                gender_c = UNKNOWN
            if gender_b == MALE:
                rel_str = self._get_male_cousin(
                    Gb - 1,
                    Ga - Gb,
                    lower=False,
                    step=step,
                    inlaw=inlaw,
                    gender_c=gender_c,
                )
            elif gender_b == FEMALE:
                rel_str = self._get_female_cousin(
                    Gb - 1,
                    Ga - Gb,
                    lower=False,
                    step=step,
                    inlaw=inlaw,
                    gender_c=gender_c,
                )
            else:
                rel_str = "%s o %s" % (
                    self._get_male_cousin(
                        Gb - 1, Ga - Gb, lower=False, step=step, inlaw=inlaw
                    ),
                    self._get_female_cousin(
                        Gb - 1, Ga - Gb, lower=False, step=step, inlaw=inlaw
                    ),
                )

        elif Gb > Ga:
            # These are cousins in different generations with the second person
            # being in a lower generation from the common ancestor than the
            # first person.
            # We need to know the gender of the person who is an ancestor of the second person and
            # is on the same generation that the first person
            if reltocommon_b[Gb - Ga - 1] == "f":
                gender_c = MALE
            elif reltocommon_b[Gb - Ga - 1] == "m":
                gender_c = FEMALE
            else:
                gender_c = UNKNOWN
            if gender_b == MALE:
                rel_str = self._get_male_cousin(
                    Ga - 1,
                    Gb - Ga,
                    lower=True,
                    step=step,
                    inlaw=inlaw,
                    gender_c=gender_c,
                )
            elif gender_b == FEMALE:
                rel_str = self._get_female_cousin(
                    Ga - 1,
                    Gb - Ga,
                    lower=True,
                    step=step,
                    inlaw=inlaw,
                    gender_c=gender_c,
                )
            else:
                rel_str = "%s o %s" % (
                    self._get_male_cousin(
                        Ga - 1, Gb - Ga, lower=True, step=step, inlaw=inlaw
                    ),
                    self._get_female_cousin(
                        Ga - 1, Gb - Ga, lower=True, step=step, inlaw=inlaw
                    ),
                )

        return rel_str

    def get_sibling_relationship_string(
        self, sib_type, gender_a, gender_b, in_law_a=False, in_law_b=False
    ):
        """ """

        rel_str = ""
        if gender_b != FEMALE:
            if sib_type == self.NORM_SIB or sib_type == self.UNKNOWN_SIB:
                rel_str = "hermano"
            elif sib_type == self.HALF_SIB_MOTHER or sib_type == self.HALF_SIB_FATHER:
                rel_str = "medio hermano"
            elif sib_type == self.STEP_SIB:
                rel_str = "hermanastro"
            if in_law_a or in_law_b:
                rel_str += " político"

        if gender_b == UNKNOWN:
            rel_str += " o "

        if gender_b != MALE:
            if sib_type == self.NORM_SIB or sib_type == self.UNKNOWN_SIB:
                rel_str += "hermana"
            elif sib_type == self.HALF_SIB_MOTHER or sib_type == self.HALF_SIB_FATHER:
                rel_str += "medio hermana"
            elif sib_type == self.STEP_SIB:
                rel_str += "hermanastra"
            if in_law_a or in_law_b:
                rel_str += " política"

        return rel_str


if __name__ == "__main__":
    # Test function. Call it as follows from the command line (so as to find
    #        imported modules):
    #    export PYTHONPATH=/path/to/gramps/src
    # python src/plugins/rel/rel_es.py
    # (Above not needed here)

    """TRANSLATORS, copy this if statement at the bottom of your
    rel_xx.py module, and test your work with:
    python src/plugins/rel/rel_xx.py
    """
    from gramps.gen.relationship import test

    RC = RelationshipCalculator()
    test(RC, True)



# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2009       Pander Musubi
# Copyright (C) 2009       Douglas S. Blank
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#
#

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
from collections import defaultdict

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.plug import Gramplet
from gramps.gen.config import config
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

_YIELD_INTERVAL = 350


def make_tag_size(n, counts, mins=8, maxs=20):
    # return font sizes mins to maxs
    diff = maxs - mins
    # based on counts (biggest to smallest)
    if len(counts) > 1:
        position = diff - (diff * (float(counts.index(n)) / (len(counts) - 1)))
    else:
        position = 0
    return int(position) + mins


class GivenNameCloudGramplet(Gramplet):
    def init(self):
        self.set_tooltip(_("Double-click given name for details"))
        self.top_size = 100  # will be overwritten in load
        self.set_text(_("No Family Tree loaded."))

    def db_changed(self):
        self.connect(self.dbstate.db, "person-add", self.update)
        self.connect(self.dbstate.db, "person-delete", self.update)
        self.connect(self.dbstate.db, "person-update", self.update)

    def on_load(self):
        if len(self.gui.data) > 0:
            self.top_size = int(self.gui.data[0])

    def on_save(self):
        self.gui.data = [self.top_size]

    def main(self):
        self.set_text(_("Processing...") + "\n")
        yield True
        givensubnames = defaultdict(int)
        representative_handle = {}

        cnt = 0
        for person in self.dbstate.db.iter_people():
            allnames = [person.get_primary_name()] + person.get_alternate_names()
            allnames = set(name.get_first_name().strip() for name in allnames)
            for givenname in allnames:
                nbsp = givenname.split("\u00a0")
                if len(nbsp) > 1:  # there was an NBSP, a non-breaking space
                    first_two = nbsp[0] + "\u00a0" + nbsp[1].split()[0]
                    givensubnames[first_two] += 1
                    representative_handle[first_two] = person.handle
                    givenname = " ".join(nbsp[1].split()[1:])
                for givensubname in givenname.split():
                    givensubnames[givensubname] += 1
                    representative_handle[givensubname] = person.handle
            cnt += 1
            if not cnt % _YIELD_INTERVAL:
                yield True

        total_people = cnt
        givensubname_sort = []

        total = cnt = 0
        for givensubname in givensubnames:
            givensubname_sort.append((givensubnames[givensubname], givensubname))
            total += givensubnames[givensubname]
            cnt += 1
            if not cnt % _YIELD_INTERVAL:
                yield True

        total_givensubnames = cnt
        givensubname_sort.sort(reverse=True)
        cloud_names = []
        cloud_values = []

        for count, givensubname in givensubname_sort:
            cloud_names.append((count, givensubname))
            cloud_values.append(count)

        cloud_names.sort(key=lambda k: k[1])
        counts = sorted(set(cloud_values), reverse=True)
        line = 0
        ### All done!
        # Now, find out how many we can display without going over top_size:
        totals = defaultdict(int)
        for count, givensubname in cloud_names:  # givensubname_sort:
            totals[count] += 1
        sums = sorted(totals, reverse=True)
        total = 0
        include_greater_than = 0
        for s in sums:
            if total + totals[s] <= self.top_size:
                total += totals[s]
            else:
                include_greater_than = s
                break
        # Ok, now we can show those counts > include_greater_than:

        self.set_text("")
        showing = 0
        for count, givensubname in cloud_names:  # givensubname_sort:
            if count > include_greater_than:
                if len(givensubname) == 0:
                    text = config.get("preferences.no-surname-text")
                else:
                    text = givensubname
                size = make_tag_size(count, counts)
                self.link(
                    text,
                    "Given",
                    text,
                    size,
                    "%s, %.2f%% (%d)"
                    % (text, (float(count) / total_people) * 100, count),
                )
                self.append_text(" ")
                showing += 1

        self.append_text(
            ("\n\n" + _("Total unique given names") + ": %d\n") % total_givensubnames
        )
        self.append_text((_("Total given names showing") + ": %d\n") % showing)
        self.append_text((_("Total people") + ": %d") % total_people, "begin")



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2005-2007  Donald N. Allingham
# Copyright (C) 2008       Brian G. Matherly
# Copyright (C) 2009       Benny Malengier
# Copyright (C) 2010       Nick Hall
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# GNOME modules
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
from gi.repository import Gio

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.config import config
from gramps.gui.basesidebar import BaseSidebar
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext


# -------------------------------------------------------------------------
#
# DropdownSidebar class
#
# -------------------------------------------------------------------------
class DropdownSidebar(BaseSidebar):
    """
    A sidebar displaying toggle buttons and buttons with drop-down menus that
    allows the user to change the current view.
    """

    def __init__(self, dbstate, uistate, categories, views):
        self.viewmanager = uistate.viewmanager
        self.views = views

        self.buttons = []
        self.button_handlers = []

        self.window = Gtk.ScrolledWindow()
        grid = Gtk.Grid()
        self.window.add(grid)
        self.window.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        self.window.show()

        use_text = config.get("interface.sidebar-text")
        for cat_num, cat_name, cat_icon in categories:
            self.__make_category(grid, use_text, cat_num, cat_name, cat_icon)

        grid.show_all()

        action_group = Gio.SimpleActionGroup()
        action_group.add_action_entries(
            [
                ("select", self.__menu_selected, "(ii)"),
            ],
            None,
        )
        self.window.insert_action_group("sidebar", action_group)

    def get_top(self):
        """
        Return the top container widget for the GUI.
        """
        return self.window

    def view_changed(self, cat_num, view_num):
        """
        Called when the active view is changed.
        """
        # Set new button as selected
        self.__handlers_block()
        for index, button in enumerate(self.buttons):
            if index == cat_num:
                button.set_active(True)
            else:
                button.set_active(False)
        self.__handlers_unblock()

    def __handlers_block(self):
        """
        Block signals to the buttons to prevent spurious events.
        """
        for idx, button in enumerate(self.buttons):
            button.handler_block(self.button_handlers[idx])

    def __handlers_unblock(self):
        """
        Unblock signals to the buttons.
        """
        for idx, button in enumerate(self.buttons):
            button.handler_unblock(self.button_handlers[idx])

    def __category_clicked(self, button, cat_num):
        """
        Called when a category button is clicked.
        """
        # Make the button active.  If it was already active the category will
        # not change.
        button.set_active(True)
        self.viewmanager.goto_page(cat_num, None)

    def __make_menu(self, cat_num):
        """
        Called when a view drop-down arrow is clicked.
        """
        menu = Gio.Menu()
        for view_num, label, icon in self.views[cat_num]:
            menuitem = Gio.MenuItem.new(
                label=label, detailed_action=f"sidebar.select(({cat_num}, {view_num}))"
            )
            menuitem.set_icon(Gio.ThemedIcon.new(iconname=icon))
            menu.append_item(menuitem)
        return menu

    def __menu_selected(self, action, parameter, data):
        """
        Called when a view is selected from a drop-down menu.
        """
        cat_num = parameter.get_child_value(0).get_int32()
        view_num = parameter.get_child_value(1).get_int32()
        self.viewmanager.goto_page(cat_num, view_num)

    def __make_category(self, grid, use_text, cat_num, cat_name, cat_icon):
        """
        Create a row in the sidebar for a category.
        """
        # create the button
        button = Gtk.ToggleButton()
        button.set_relief(Gtk.ReliefStyle.NONE)
        self.buttons.append(button)

        # create the drop-down button to display views
        if len(self.views[cat_num]) > 1:
            dropdown = Gtk.MenuButton(use_popover=False)
            dropdown.set_relief(Gtk.ReliefStyle.NONE)
            dropdown.set_tooltip_text(_("Click to select a view"))
            menu = self.__make_menu(cat_num)
            dropdown.set_menu_model(menu)
            grid.attach(dropdown, 1, cat_num, 1, 1)
            dropdown.set_align_widget(grid)

        # add the tooltip
        button.set_tooltip_text(cat_name)

        # connect the signal, along with the cat_num as user data
        handler_id = button.connect("clicked", self.__category_clicked, cat_num)
        self.button_handlers.append(handler_id)
        button.show()

        # add the image. If we are using text, use the BUTTON (larger) size.
        # otherwise, use the smaller size
        hbox = Gtk.Box()
        hbox.show()
        image = Gtk.Image()
        if use_text:
            image.set_from_icon_name(cat_icon, Gtk.IconSize.BUTTON)
        else:
            image.set_from_icon_name(cat_icon, Gtk.IconSize.DND)
        image.show()
        hbox.pack_start(image, False, False, 0)
        hbox.set_spacing(4)

        # add text if requested
        if use_text:
            label = Gtk.Label(label=cat_name)
            label.show()
            hbox.pack_start(label, False, True, 0)

        button.add(hbox)

        # Enable view switching during DnD
        button.drag_dest_set(0, [], 0)
        button.connect("drag_motion", self.cb_switch_page_on_dnd, cat_num)

        grid.attach(button, 0, cat_num, 1, 1)

    def cb_switch_page_on_dnd(self, widget, context, xpos, ypos, time, page_no):
        """
        Switches the page based on drag and drop.
        """
        self.__handlers_block()
        if self.viewmanager.notebook.get_current_page() != page_no:
            self.viewmanager.notebook.set_current_page(page_no)
        self.__handlers_unblock()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
# Copyright (C) 2010  Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .. import MatchesEventFilterBase


# -------------------------------------------------------------------------
#
# MatchesFilter
#
# -------------------------------------------------------------------------
class MatchesEventFilter(MatchesEventFilterBase):
    """
    Rule that checks against another filter.

    This is a base rule for subclassing by specific objects.
    Subclasses need to define the namespace class attribute.

    """

    labels = [_("Event filter name:")]
    name = _("Persons with events matching the <event filter>")
    description = _(
        "Matches persons who have events that match a certain" " event filter"
    )
    category = _("Event filters")

    # we want to have this filter show event filters
    namespace = "Event"



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2008  Brian G. Matherly
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .._hasgallerybase import HasGalleryBase


# -------------------------------------------------------------------------
# "People with media object reference "
# -------------------------------------------------------------------------
class HavePhotos(HasGalleryBase):
    """Rule that checks for person who has media object reference"""

    name = _("People with <count> media")
    description = _("Matches people with a certain number of items in the gallery")

    def __init__(self, arg, use_regex=False, use_case=False):
        # Upgrade from pre 3.1 HasPhotos filter, use defaults that correspond
        # Previous filter had 0 arguments
        if len(arg) == 0:
            HasGalleryBase.__init__(self, ["0", "greater than"], use_regex, use_case)
        else:
            HasGalleryBase.__init__(self, arg, use_regex, use_case)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#
# gen.filters.rules/Person/_IsDuplicatedAncestorOf.py

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .. import Rule


# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from typing import Set
from ....lib import Person
from ....db import Database


# -------------------------------------------------------------------------
#
# IsDuplicatedAncestorOf
#
# -------------------------------------------------------------------------
class IsDuplicatedAncestorOf(Rule):
    """Rule that checks for a person that is a duplicated ancestor of
    a specified person"""

    labels = [_("ID:")]
    name = _("Duplicated ancestors of <person>")
    category = _("Ancestral filters")
    description = _(
        "Matches people that are ancestors twice or more " "of a specified person"
    )

    def prepare(self, db: Database, user):
        self.db = db
        self.cache: Set[str] = set()
        self.selected_handles: Set[str] = set()
        root_person = db.get_person_from_gramps_id(self.list[0])
        if root_person:
            self.init_ancestor_list(db, root_person)

    def reset(self):
        self.cache.clear()
        self.selected_handles.clear()

    def apply_to_one(self, db: Database, person: Person) -> bool:
        return person.handle in self.selected_handles

    def init_ancestor_list(self, db: Database, person: Person):
        fam_id = (
            person.parent_family_list[0] if len(person.parent_family_list) > 0 else None
        )
        if fam_id:
            fam = db.get_family_from_handle(fam_id)
            if fam:
                f_id = fam.father_handle
                m_id = fam.mother_handle
                if m_id:
                    self.go_deeper(db, db.get_person_from_handle(m_id))
                if f_id:
                    self.go_deeper(db, db.get_person_from_handle(f_id))

    def go_deeper(self, db: Database, person: Person):
        if person and person.handle in self.cache:
            self.selected_handles.add((person.handle))
            # the following keeps from scanning same parts of tree multiple
            # times and avoids crash on tree loops.
            return
        self.cache.add(person.handle)
        self.init_ancestor_list(db, person)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2010  Doug Blank <doug.blank@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# python modules
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
import re

# -------------------------------------------------------------------------
#
# gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext
from ..managedwindow import ManagedWindow
from ..display import display_help
from ..glade import Glade
from gramps.gen.simple import SimpleAccess
from gramps.gen.const import URL_MANUAL_SECT2

# -------------------------------------------------------------------------
#
# Constants
#
# -------------------------------------------------------------------------

WIKI_HELP_PAGE = URL_MANUAL_SECT2
WIKI_HELP_SEC = _("Link_Editor", "manual")

WEB, EVENT, FAMILY, MEDIA, NOTE, PERSON, PLACE, REPOSITORY, SOURCE, CITATION = list(
    range(10)
)
OBJECT_MAP = {
    EVENT: "Event",
    FAMILY: "Family",
    MEDIA: "Media",
    NOTE: "Note",
    PERSON: "Person",
    PLACE: "Place",
    REPOSITORY: "Repository",
    SOURCE: "Source",
    CITATION: "Citation",
}


# -------------------------------------------------------------------------
#
# EditLink class
#
# -------------------------------------------------------------------------
class EditLink(ManagedWindow):
    def __init__(self, dbstate, uistate, track, url, callback):
        self.url = url
        self.dbstate = dbstate
        self.simple_access = SimpleAccess(self.dbstate.db)
        self.callback = callback

        ManagedWindow.__init__(self, uistate, track, url, modal=True)

        self._local_init()
        self._connect_signals()
        self.show()

    def _local_init(self):
        self.top = Glade()
        self.set_window(
            self.top.toplevel, self.top.get_object("title"), _("Link Editor")
        )
        self.setup_configs("interface.editlink", 600, 160)

        self.uri_list = self.top.get_object("link_type")
        for text in [
            _("Internet Address"),  # 0 this order range above
            _("Event"),  # 1
            _("Family"),  # 2
            _("Media"),  # 3
            _("Note"),  # 4
            _("Person"),  # 5
            _("Place"),  # 6
            _("Repository"),  # 7
            _("Source"),  # 8
            _("Citation"),  # 9
        ]:
            self.uri_list.append_text(text)
        self.pick_item = self.top.get_object("button1")
        self.new_button = self.top.get_object("new")
        self.edit_button = self.top.get_object("edit")
        self.selected = self.top.get_object("label1")
        self.url_link = self.top.get_object("entry1")
        self.uri_list.connect("changed", self._on_type_changed)
        self.pick_item.connect("clicked", self._on_pick_one)
        self.new_button.connect("clicked", self._on_new)
        self.edit_button.connect("clicked", self._on_edit_one)
        if self.url.startswith("gramps://"):
            object_class, prop, value = self.url[9:].split("/", 2)
            if object_class == "Event":
                self.uri_list.set_active(EVENT)
            elif object_class == "Family":
                self.uri_list.set_active(FAMILY)
            elif object_class == "Media":
                self.uri_list.set_active(MEDIA)
            elif object_class == "Note":
                self.uri_list.set_active(NOTE)
            elif object_class == "Person":
                self.uri_list.set_active(PERSON)
            elif object_class == "Place":
                self.uri_list.set_active(PLACE)
            elif object_class == "Repository":
                self.uri_list.set_active(REPOSITORY)
            elif object_class == "Source":
                self.uri_list.set_active(SOURCE)
            elif object_class == "Citation":
                self.uri_list.set_active(CITATION)
            # set texts:
            self.selected.set_text(self.display_link(object_class, prop, value))
            self.url_link.set_text("gramps://%s/%s/%s" % (object_class, prop, value))
        else:
            self.uri_list.set_active(WEB)
            self.url_link.set_text(self.url)
        self.url_link.connect("changed", self.update_ui)

    def update_ui(self, widget):
        url = self.url_link.get_text()
        # text needs to have 3 or more chars://and at least one char
        # valid internet address are: http://, https://, ftp:// and mailto:
        match = re.match(r"(\w{3,}://\w+|mailto:\w+)", url)
        if match:
            self.ok_button.set_sensitive(True)
        else:
            self.ok_button.set_sensitive(False)

    def display_link(self, obj_class, prop, value):
        return self.simple_access.display(obj_class, prop, value)

    def _on_new_callback(self, obj):
        object_class = obj.__class__.__name__
        self.selected.set_text(self.display_link(object_class, "handle", obj.handle))
        self.url_link.set_text(
            "gramps://%s/%s/%s" % (object_class, "handle", obj.handle)
        )

    def _on_new(self, widget):
        from ..editors import EditObject

        object_class = OBJECT_MAP[self.uri_list.get_active()]
        EditObject(
            self.dbstate,
            self.uistate,
            self.track,
            object_class,
            callback=self._on_new_callback,
        )

    def _on_edit_one(self, widget):
        from ..editors import EditObject

        uri = self.url_link.get_text()
        if uri.startswith("gramps://"):
            obj_class, prop, value = uri[9:].split("/", 2)
            EditObject(self.dbstate, self.uistate, self.track, obj_class, prop, value)

    def _on_pick_one(self, widget):
        from ..selectors import SelectorFactory

        object_class = OBJECT_MAP[self.uri_list.get_active()]
        Select = SelectorFactory(object_class)
        uri = self.url_link.get_text()
        default = None
        if uri.startswith("gramps://"):
            obj_class, prop, value = uri[9:].split("/", 2)
            if object_class == obj_class:
                if prop == "handle":
                    default = value
                elif prop == "gramps_id" and object_class in OBJECT_MAP.values():
                    person = self.dbstate.db.method(
                        "get_%s_from_gramps_id", object_class
                    )(value)
                    if person:
                        default = person.handle
        d = Select(self.dbstate, self.uistate, self.track, default=default)

        result = d.run()
        if result:
            prop = "handle"
            value = result.handle
            self.selected.set_text(self.display_link(object_class, prop, value))
            self.url_link.set_text("gramps://%s/%s/%s" % (object_class, prop, value))

    def _on_type_changed(self, widget):
        if self.uri_list.get_active() == WEB:
            self.url_link.set_sensitive(True)
            self.pick_item.set_sensitive(False)
            self.new_button.set_sensitive(False)
            self.edit_button.set_sensitive(False)
            self.selected.set_text("")
            self.url_link.set_text("https://")
        else:
            self.url_link.set_sensitive(False)
            self.pick_item.set_sensitive(True)
            self.new_button.set_sensitive(True)
            self.edit_button.set_sensitive(True)
            object_class = OBJECT_MAP[self.uri_list.get_active()]
            handle = self.uistate.get_active(object_class)
            if handle:
                self.selected.set_text(
                    self.display_link(object_class, "handle", handle)
                )
                self.url_link.set_text("gramps://%s/handle/%s" % (object_class, handle))
            else:
                self.selected.set_text("")
                self.url_link.set_text("")

    def get_uri(self):
        if self.uri_list.get_active() == WEB:
            return self.url_link.get_text()
        else:
            return self.url_link.get_text()

    def _connect_signals(self):
        self.define_cancel_button(self.top.get_object("button125"))
        self.ok_button = self.top.get_object("button124")
        self.define_ok_button(self.ok_button, self.save)
        # TODO help button (rename glade button name)
        self.define_help_button(
            self.top.get_object("button130"), WIKI_HELP_PAGE, WIKI_HELP_SEC
        )
        self.update_ui(self.url_link)

    def build_menu_names(self, obj):
        etitle = _("Link Editor")
        return (etitle, etitle)

    def define_ok_button(self, button, function):
        button.connect("clicked", function)

    def save(self, widget):
        self.callback(self.get_uri())
        self.close()

    def define_cancel_button(self, button):
        button.connect("clicked", self.close)

    def define_help_button(self, button, webpage="", section=""):
        button.connect("clicked", lambda x: display_help(webpage, section))



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2008  Brian G. Matherly
# Copyright (C) 2008  Jerome Rapinat
# Copyright (C) 2008  Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#
# gen.filters.rules/Family/_HasGallery.py

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .._hasgallerybase import HasGalleryBase


# -------------------------------------------------------------------------
# "Families who have media object reference"
# -------------------------------------------------------------------------
class HasGallery(HasGalleryBase):
    """Rule that checks for family who has media object reference"""

    name = _("Families with <count> media")
    description = _("Matches families with a certain number of items in the gallery")



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
# Copyright (C) 2015       Nick Hall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.
#

# -------------------------------------------------------------------------
#
# Python classes
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext
from gi.repository import GObject, GLib

# -------------------------------------------------------------------------
#
# Gramps classes
#
# -------------------------------------------------------------------------
from gramps.gen.lib import PlaceName
from gramps.gen.errors import WindowActiveError
from ...ddtargets import DdTargets
from .placenamemodel import PlaceNameModel
from .embeddedlist import EmbeddedList, TEXT_COL, MARKUP_COL, ICON_COL


# -------------------------------------------------------------------------
#
# PlaceNameEmbedList
#
# -------------------------------------------------------------------------
class PlaceNameEmbedList(EmbeddedList):
    _HANDLE_COL = 3
    _DND_TYPE = DdTargets.PLACENAME

    _MSG = {
        "add": _("Create and add a new place name"),
        "del": _("Remove the existing place name"),
        "edit": _("Edit the selected place name"),
        "up": _("Move the selected place name upwards"),
        "down": _("Move the selected place name downwards"),
    }

    # index = column in model. Value =
    #  (name, sortcol in model, width, markup/text, weigth_col
    _column_names = [
        (_("Name"), 0, 250, TEXT_COL, -1, None),
        (_("Date"), 1, 250, TEXT_COL, -1, None),
        (_("Language"), 2, 100, TEXT_COL, -1, None),
    ]

    def __init__(self, dbstate, uistate, track, data, config_key):
        self.data = data
        EmbeddedList.__init__(
            self,
            dbstate,
            uistate,
            track,
            _("Alternative Names"),
            PlaceNameModel,
            config_key,
            move_buttons=True,
        )

    def get_data(self):
        return self.data

    def column_order(self):
        return ((1, 0), (1, 1), (1, 2))

    def add_button_clicked(self, obj):
        """
        Called when the Add button is clicked.
        """
        pname = PlaceName()
        try:
            from .. import EditPlaceName

            EditPlaceName(
                self.dbstate, self.uistate, self.track, pname, self.add_callback
            )
        except WindowActiveError:
            return

    def add_callback(self, pname):
        """
        Called to update the screen when a new place name is added.
        """
        data = self.get_data()
        data.append(pname)
        self.rebuild()
        GLib.idle_add(self.tree.scroll_to_cell, len(data) - 1)

    def edit_button_clicked(self, obj):
        """
        Called with the Edit button is clicked.
        """
        pname = self.get_selected()
        if pname:
            try:
                from .. import EditPlaceName

                EditPlaceName(
                    self.dbstate, self.uistate, self.track, pname, self.edit_callback
                )
            except WindowActiveError:
                return

    def edit_callback(self, name):
        """
        Called to update the screen when the place name changes.
        """
        self.rebuild()
